// Example code for homework 2.

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "levelset.h"
#include "particles.h"

// global parameters for simulation
int end_seconds; // final time
const float frames_per_second=24;
int new_particles_per_frame;
float min_particle_radius;
float max_particle_radius;
float particle_density;
float drag;
float base_wind[3];
float epsilon; // coefficient of restitution for elastic cases
float friction;
float dump_width;
float max_dt; // you'll have to figure this one out

// global variables
int frame=0;
float sim_t=0; // current time

// functions
void read_parameters(const char *filename);
void calculate_max_dt();
void add_new_frame_particles(ParticleSystem *ps);
void advance_with_collisions(const LevelSet *ls, ParticleSystem *ps, float end_time);
void process_collision(const float n[3], float v[3], float coefficient_of_restitution);
void advance_until(const float startx[3], const float startv[3], float m, float r,
                   float start_t, float end_t, float endx[3], float endv[3]);
void advance_one_step(float x[3], float v[3], float m, float r,  float t, float dt);
void force(const float x[3], const float v[3], float m, float r, float t, float f[3]);

// start of definitions
int main(int argc, char **argv)
{
   LevelSet ls;
   ParticleSystem ps;
   const char *output_directory=".";

   if(argc<3 || argc>4){
      fprintf(stderr,"Need arguments: parameter-file levelset-file [optional-output-directory]\n");
      return 1;
   }else if(argc==4)
      output_directory=argv[3];

   read_parameters(argv[1]);
   calculate_max_dt();
   read_level_set(argv[2], &ls);

   init_particles(&ps);
   add_new_frame_particles(&ps);
   write_particles(&ps, "%s/particles%04d", output_directory, frame);

   while(frame/frames_per_second < end_seconds){
      ++frame;
      advance_with_collisions(&ls, &ps, frame/frames_per_second);
      sim_t=frame/frames_per_second;
      add_new_frame_particles(&ps);
      write_particles(&ps, "%s/particles%04d", output_directory, frame);
      fputc(frame%10 ? '+' : '#', stdout); fflush(stdout);
   }
   return 0;
}

void read_parameters(const char *filename)
{
   FILE *fp=fopen(filename,"rt");
   fscanf(fp, "%d", &end_seconds);
   fscanf(fp, "%d", &new_particles_per_frame);
   fscanf(fp, "%f %f", &min_particle_radius, &max_particle_radius);
   fscanf(fp, "%f", &particle_density);
   fscanf(fp, "%f", &drag);
   fscanf(fp, "%f %f %f", base_wind, base_wind+1, base_wind+2);
   fscanf(fp, "%f %f", &epsilon, &friction);
   fscanf(fp, "%f", &dump_width);
}

void calculate_max_dt()
{
   // @@@ fill in this part
}

void add_new_frame_particles(ParticleSystem *ps)
{
   // @@@ fill in code to add new_particles_per_frame to the system at appropriate random locations
   // add_particle() from particles.h may be useful!
}

void advance_with_collisions(const LevelSet *ls, ParticleSystem *ps, float end_t)
{
   // @@@ fill in code here that loops over the particles, advancing each to end_t.
   // Each particle should use advance_until(), then do elastic collisions, then inelastic contact.
}

void process_collision(const float n[3], float v[3], float coefficient_of_restitution)
{
   // @@@ fill in code that takes a normal vector n, input velocity v, and given coefficient of
   // restitution (maybe not epsilon - could be smaller for collision processing algorithm)
   // and updates v to after the frictional collision. 
}

void advance_until(const float startx[3], const float startv[3], float m, float r, float start_t, float end_t, float endx[3], float endv[3])
{
   // @@@ fill in code that advances a particle from time start_t at position startx and velocity startv (and
   // with mass m and radius r) to end_t. Store the final position and velocity in endx and endv.
   // Obey the time step limit max_dt - you may need to take many smaller time steps to get to end_t.
}

void advance_one_step(float x[3], float v[3], float m, float r,  float t, float dt)
{
   // @@@ fill in code for doing one time step of Symplectic Euler
   // Update x and v in place, given mass m, radius r, time t at start of the step, and time step size dt.
}

void force(const float x[3], const float v[3], float m, float r, float t, float f[3])
{
   // @@@ fill in your code for the force function (gravity and wind drag).
   // Save the force vector in f, given the position x, velocity v, mass m, radius r, and time t.
}

